import random
import math 

from pymunk.vec2d import vec2d

from Sound import gSound


class Actor:
    def __init__(self, id, type):
        self.id = id
        self.type = type
        self.creationData = {}
        
        # the actor's transform
        self.node = None
        self.color = None
        self.baseColor = None
        
        # the phsyics shapes and body
        self.shapes = []
        self.body = None
        self.static = False
        
        self.collidedLastFrame = False
        
        self.removable = False
        # game specific stuff
        self.bounce = 1.0
        
        self.animFactor = 0.0
        self.animTime = 0.0
        
        self.lastContacts = []
        
        self.staticTranslation = None
        
    def getPos(self):
        return vec2d( self.node.translate[0], self.node.translate[1] )
        
    def update(self, dt):
        if self.node is not None and self.body is not None:
            if not self.static:
                x = self.body.contents.p.x
                y = self.body.contents.p.y
                
                self.node.translate = ( x, y, 0)
                v = self.body.contents.rot
                self.node.angle = v.get_angle()
            else:
                
                if self.staticTranslation is None:
                    x,y,z = self.node.translate
                    self.staticTranslation = x,y,z
                
                x,y,z = self.staticTranslation
                # y = self.body.contents.p.y
                
                if self.animFactor > 0.0:
                    x += self.animFactor * math.sin( self.animTime * 50 )
                    y += self.animFactor * math.cos( self.animTime * 50 )
                    self.animFactor -= dt * 10
                    self.animTime += dt
                
                self.node.translate = ( x, y, 0)
                
            
#            if self.color is not None:
#                if self.collidedLastFrame: 
#                    self.color.rgba = 0.5,0.5,0.5,1
#                else:
#                    self.color.rgba = self.baseColor
            
#            for c in self.lastContacts:
#                #print dir(c)
#                energy = c.jnAcc#.get_length()
#                if energy > 200:
#                    print "collision", energy
#                    gSound.playSound(self, "metalbounce")
#                    break 
            self.lastContacts = []
            self.collidedLastFrame = False
    
    def onCollide( self, shape2, contact_points, num_contact_points, data):
        #global gSound
        self.collidedLastFrame = True
        
#        print "--coll--"
#        self.lastContacts = contact_points
#        
#        print "coll", len(contact_points)
#        if len(contact_points) > 2:
#             gSound.playSound(self, "metalbounce")
            
#        for c in contact_points:
#            #print dir(c)
#            energy = c.jnAcc#.get_length()
#            if energy > 200:
#                print "collision", energy
#                gSound.playSound(self, "metalbounce")
#                break 
        
        #v = self.body.contents.v.get_length()
    
    
    def save(self):
        #print self
        #pos = "%i,%i" % (self.getPos().x, self.getPos().y )
        #print self.type, pos
        return {"type":self.type, "pos":self.getPos(), "data":self.creationData}
    
    
        