import Actor, BallActor, BallSpawnerActor, GoalActor, GravityActor, StretchActor
from pymunk.vec2d import vec2d
import pymunk.util
import data

import random

class ActorManager:
    
    def __init__(self, graphics, physics, gamestate ):
        self.id = 0
        self.graphics = graphics
        self.physics = physics
        self.actors = []
        self.actors_to_remove = set()
        self.gameState = gamestate
        self.shape_to_actor_map = {}
        
    def update(self, dt):
        
        for actor in self.actors:
            actor.update( dt )
            x,y,z = actor.node.translate
            if x < -2000 or x > 2000 or y < -2000 or y > 2000:
                self.actors_to_remove.add((actor, None))

        for actor, reason in self.actors_to_remove:
            if actor.type == "ball" and reason == "goal":
                self.gameState.level.addFinishedBall()
            self.destroyActor(actor)
            
        self.actors_to_remove = set()
        
            
    def createActor(self, type, pos, free_object = False, **data):
        
        if free_object and type in self.gameState.level.objects_left:
            if self.gameState.level.objects_left[type] < 1:
                return
            else:
                self.gameState.level.objects_left[type] -= 1
            
        #print "AM: Creating actor of type", type, "at", pos, "with data", data
        if type == "ball":
            actor = BallActor.BallActor( self.id, type, self.physics )
        elif type == "ballspawner":
            actor = BallSpawnerActor.BallSpawnerActor( self.id, type, self.physics, self )
        elif type == "goal":
            actor = GoalActor.GoalActor( self.id, type, self.physics, self )
        elif type == "gravity1" or type == "gravity2":
            actor = GravityActor.GravityActor( self.id, type, self.physics, self )
        elif type == "stretch":
            actor = StretchActor.StretchActor( self.id, type, self.physics)
        else:
            actor = Actor.Actor( self.id, type )
        
        actor.removable = free_object    
        actor.creationData = data
        actor.node = self.graphics.createGroup(type )
        actor.node.translate = ( pos.x, pos.y, 0.0 )
        ### Create graphics
        r,g,b = (1,1,1) 
        if not data.has_key("static") or data["static"]:
            r,g,b = (1,1,1)
        actor.baseColor = (r,b,g,1)
        actor.color = self.graphics.createColor( r,g,b )
        if type == "stretch":
            #print "stretchpos:", pos
            actor.static = data["static"]
            ### Create graphics
            texture = self.graphics.createTexture( "stretch" )
            quad = self.graphics.createQuad( (1, 1) )
            actor.quad = quad
            #quad.vertices = 
            actor.node.add( actor.color, texture, quad )
            actor.node.accept( self.graphics.getCompiler() )
        
        if type == "box":
            actor.static = data["static"]
            size = data["size"]
            
            ### Generate points
            line_points = (-size.x,-size.y),(-size.x,size.y),(size.x,size.y),(size.x,-size.y),(-size.x,-size.y)
            line_points = map(vec2d, line_points)

            self.addPolygon( actor, line_points, pos, e = 0.9)
 
        if type == "polygon":
            actor.static = data["static"]
            line_points  = data["points"]
            #line_points = pymunk.util.reduce_poly(line_points)
            line_points = pymunk.util.convex_hull( line_points )
            offset = pymunk.util.average( line_points )
            pos += offset
            
            self.addPolygon( actor, line_points, pos)
            
        if type == "ball":
            body, shape = self.physics.createBall( pos )               
            actor.shapes = [shape]
            actor.body = body
            
            ### Create graphics
            texture = self.graphics.createTexture( "ball" )
            quad = self.graphics.createQuad( (32, 32) )
            actor.node.add( texture )
            actor.node.add( quad )
            actor.node.accept( self.graphics.getCompiler() )
            
        if type == "ballspawner":
            ### Create graphics
            texture = self.graphics.createTexture( "ballspawner" )
            quad = self.graphics.createQuad( (128, 128) )
            actor.node.add( texture )
            actor.node.add( quad )
            actor.node.accept( self.graphics.getCompiler() )
        
        if type == "goal":
            #body, shape = self.physics.createBall( pos, 1.0, 150.0, True )               
            #actor.shapes = [shape]
            #actor.body = body
            actor.static = True
            ### Generate points
            size = vec2d(50, 50)
            line_points = (-size.x,-size.y),(-size.x,size.y),(size.x,size.y),(size.x,-size.y),(-size.x,-size.y)
            line_points = map(vec2d, line_points)

            self.addPolygon( actor, line_points, pos, 0, False)
            
            ### Create graphics
            texture = self.graphics.createTexture( "goal" )
            quad = self.graphics.createQuad( (100, 100) )
            actor.node.add( texture )
            actor.node.add( quad )
            actor.node.accept( self.graphics.getCompiler() )
            
            
        if type == "gravity1" or type == "gravity2":
            #body, shape = self.physics.createBall( pos, 1.0, 150.0, True )               
            #actor.shapes = [shape]
            #actor.body = body
            actor.static = True
            ### Generate points
            s = vec2d(30, 30)
            
            line_points = [(-s.x, s.y/3), (-s.x/3, s.y), (s.x/3, s.y), (s.x, s.y/3),
                          (s.x, -s.y/3), (s.x/3, -s.y), (-s.x/3, -s.y), (-s.x, -s.y/3)]
            line_points = map(vec2d, line_points)

            self.addPolygon( actor, line_points, pos, 0, False)
            
            ### Create graphics
            texture = self.graphics.createTexture( type )
            quad = self.graphics.createQuad( (64, 64) )
            actor.node.add( texture )
            actor.node.add( quad )
            actor.node.accept( self.graphics.getCompiler() )
            
            
        if actor.body != None:
            actor.body.contents.p = pos
            #self.physics.setAngle(actor.body, random.randint(0,10))
            pass
 
        #actor.node.accept( self.graphics.getCompiler() )
        #self.graphics.compileAll()
            
        self.actors.append( actor )
        for s in actor.shapes:
            self.shape_to_actor_map[s.contents.id] = actor
        self.id += 1
        
        return actor
 
    def finishStretching(self, actor, vertices = None, texture_coords = None):
        line_points = []
        x,y,z = actor.node.translate
        node_pos = vec2d(x,y)
        if vertices == None:
            avg = vec2d(0,0)
            for p in actor.quad.vertices:
                pv = vec2d(p)
                avg.x += pv.x
                avg.y += pv.y
            n = len(actor.quad.vertices)
            avg /= n
            node_pos = node_pos + avg
            actor.node.translate = node_pos.x, node_pos.y, z
            v1,v2,v3,v4 = actor.quad.vertices
            actor.quad.vertices = v1 - avg , v2 - avg, v3 - avg, v4 - avg
            actor.node.accept( self.graphics.compiler )
        else:
            actor.quad.vertices = vertices
            actor.quad.texture_coords = texture_coords
            actor.node.accept( self.graphics.compiler )
        for p in actor.quad.vertices:
            pv = vec2d(p)
            #pv += avg
            line_points.append(pv + node_pos)
        line_points.append(line_points[0])    
        ### Create physics
        if actor.static:
            #offset = pymunk.util.average( line_points )
            #pos += offset
            body, shapes = self.physics.createStaticPolyLine( line_points ) #, offset )               
            actor.shapes = shapes
        else:
            body, shape = self.physics.createPolygon( line_points )
            actor.shapes = [shape]
        actor.body = body
        for s in actor.shapes:
            self.shape_to_actor_map[s.contents.id] = actor
            
        
    def addPolygon(self, actor, line_points, pos, e = 0, graphics = True):
        if not  pymunk.util.is_clockwise(line_points):
            line_points.reverse()
        
        
        ### Create graphics
        if graphics:
            line  = self.graphics.createPolyline( line_points )
            actor.node.add( actor.color )
            actor.node.add( line )
        
        ### Create physics
        if actor.static:
            body, shapes = self.physics.createStaticPolyLine( line_points, pos, e )               
            actor.shapes = shapes
        else:
            body, shape = self.physics.createPolygon( line_points )
            actor.shapes = [shape]
            
            
        actor.body = body
        
    def destroyActor(self, actor):
        #print "AM: Removing actor", actor
        
        self.graphics.groups[actor.type].remove( actor.node )
        
        if (actor.static):
            self.physics.removeStatic( actor.body, actor.shapes )    
        else:
            self.physics.remove( actor.body, actor.shapes )    
        
        for s in actor.shapes:
            del self.shape_to_actor_map[s.contents.id]
        self.actors.remove( actor )
        

        
    def findClosestActor( self, pos, max_dist ):
        min_dist = max_dist
        closest_actor = None
        for actor in self.actors:
            dist = actor.getPos().get_distance( pos )
            if dist < min_dist and dist < max_dist:
                min_dist = dist
                closest_actor = actor
            
        return closest_actor
    
    
    def saveLevel(self, level):
        
        out = [] 
        for actor in self.actors:
            if actor.type == "ball":
                continue
            
            out.append( actor.type + " " + str( actor.save() ) )
        
        out.sort()
        
        filename = "new_" + str( level ) + ".txt"
        data.save( filename, "\n".join( out ) )
        
    
    def loadLevel(self, level):
        
        filename = level + ".txt"
        
        try:
            ###rU == universal newline
            level = data.load( filename, mode="rU" )
            
            while self.actors != []:
                self.destroyActor( self.actors[-1] )
            
            for line in level.readlines():
                if line == "" or line[0] == "#":
                    continue
                
                dict_index = line.find( " " )
                
                d = eval( line[dict_index:] )
                
                type = d["type"]
                pos = d["pos"]
                actordata = d["data"]
                
                actor = self.createActor(type, pos, **actordata)
                
                if type == "stretch":
                    #actor.updateStretch( d["stretch_to"], self.graphics )
                    self.finishStretching( actor, vertices = d["vertices"], texture_coords = d["texture_coords"] )
        
        except Exception, e:
            print "Couldn't load " + filename
            print e
 
    