import pygame
from pygame.locals import *

import qgl

import data

class Graphics:
    def __init__(self):
        
        self.screenSize = (800,600)
        flags =  OPENGL|DOUBLEBUF|HWSURFACE
        pygame.display.set_mode(  self.screenSize, flags)
        
        self.compiler = qgl.render.Compiler()
        self.render = qgl.render.Render()
        self.picker = qgl.render.Picker()
        
        self.world = qgl.scene.World()
        self.gui = qgl.scene.Node()
        self.main_gui = qgl.scene.Node()
        self.gui.add( self.main_gui )
        self.mouse_pointer = qgl.scene.Group()
        self.world.foreground.add(self.mouse_pointer)
        
        self.viewport = qgl.scene.OrthoViewport( self.world, self.gui )
        self.viewport.screen_dimensions = ( 0, 0, self.screenSize[0], self.screenSize[1] )
        self.viewport.aspect = float(self.screenSize[0]) / self.screenSize[1]
               
        self.rootNode = qgl.scene.Root()
        self.rootNode.add( self.viewport )
        self.rootNode.accept( self.compiler )
  
        self.camera_speed = 500
        self.camera_zoom_speed = 1
        self.zoom = 1.0
        
        self.groups = {}
        self.textures = {}
        ### Test
        #line_points = (-3.3,-10),(-10,10),(6,15),(10,-10),(-10,-10)
        #polyline = qgl.scene.state.Polyline(line_points)
        #self.addGraphics( polyline )
    
    def setActiveBranch(self, branchName):
        self.switch.switch(branchName)
    
    def setBackground(self, levelname):
        texture = self.createTexture( "bg_" + levelname )
        quad = self.createQuad( (1000,1000) )
        self.world.background.add(texture, quad)
        self.world.background.accept( self.compiler )
        
    def moveCamera(self, v): 
        v = v * self.camera_speed
        x,y,z = self.world.camera.translate
        self.world.camera.translate = (x + v.x, y + v.y, z)
    
    def zoomCamera(self, d):
        d = 1.0 + d * self.camera_zoom_speed
        self.zoom *= d
        self.world.camera.scale = (self.zoom , self.zoom, 1)
    
    def setZoom(self, zoom):
        self.zoom = zoom
        self.world.camera.scale = zoom, zoom, 1
       
    def createGroup(self, name):
        if not name in self.groups.keys():
            group = qgl.scene.Group()
            self.world.foreground.add(group)
            self.groups[name] = group
        
        group = qgl.scene.Group()
        self.groups[name].add( group )
        
        return group
    
    def createGuiGroup(self):
        group = qgl.scene.Group()
        self.main_gui.add(group)
        return group
    
    def removeGroup(self, group):
        self.world.foreground.remove(group)
        #self.viewport.remove(group)
    
    def createColor(self, r, g, b, a=1.0):
        color = qgl.scene.state.Color((r, g, b, a))
        return color                                      
    
    def createPolyline(self, line_points):
        polyline = qgl.scene.state.Polyline(line_points)
        return polyline
    

    def createText(self, text, font):
        text = qgl.scene.state.Text(text, data.filepath(font + ".ttf"))
        return text
    
    def createTexture(self, texture_name):
        
        if not texture_name in self.textures.keys():
            #print texture_name
            texture = qgl.scene.state.Texture( data.filepath(texture_name + ".png") )
            self.textures[texture_name] = texture
        
        return self.textures[texture_name]
    
    def createQuad(self, size):
        quad = qgl.scene.state.Quad( size)
        return quad
    
    def getCompiler(self):
        return self.compiler

    def compileAll(self):
        self.rootNode.accept( self.compiler )

    #def addGraphics(self, g):
    #    #print "Graphics: Adding", g
    #    self.world.background.add( g )
    #    pass
    
    def update(self):
        #self.setActiveBranch("default")
        self.rootNode.accept( self.render )
        
    
    
    