import pygame
from pygame.locals import *
from pymunk.vec2d import vec2d

from Sound import gSound

from PauseState import PauseState
from HelpState import HelpState
from MenuState import MenuState
X, Y = 0,1

LEFT_MOUSE = 1
MIDDLE_MOUSE = 2
RIGHT_MOUSE = 3
WHEEL_UP = 4
WHEEL_DOWN = 5

class MainGameInput:
    def __init__(self, gameStateManager, graphics, physics, actorManager, gameState):
        self.actorManager = actorManager
        self.gameStateManager = gameStateManager
        self.graphics = graphics
        self.physics = physics
        self.gameState = gameState
        self.currentActor = None
        self.currentPolygon = []
        
        self.stretchActor = None
        self.actor_to_remove = None
        
        self.nextLevel = 1
        self.scrollArea = 50
        
        self.hits = []
        
        self.action = None
    def handleStretch(self, mouse_pos):
       if self.stretchActor == None:
           return
       self.stretchActor.updateStretch( self.toWorld(mouse_pos), self.graphics )
       
    def toMenu(self):    
        state = MenuState( self.gameStateManager, self.graphics )
        self.graphics.main_gui.disable()
        self.gameStateManager.pushState( state )
        
    def toHelp(self):
        self.gameStateManager.pushState( HelpState(self.gameStateManager, self.graphics) )
        
    def reset(self):
        self.gameState.reset()
        
    def update(self, dt):
        pressed = pygame.key.get_pressed()
        ### Camera and Zoom
        if pressed[K_LEFT]:
            self.graphics.moveCamera(vec2d(1,0) * dt)    
        if pressed[K_RIGHT]:
            self.graphics.moveCamera(vec2d(-1,0) * dt)    
        if pressed[K_UP]:
            self.graphics.moveCamera(vec2d(0,-1) * dt)    
        if pressed[K_DOWN]:
            self.graphics.moveCamera(vec2d(0,1) * dt)    
        if pressed[K_PAGEUP]:
            self.graphics.zoomCamera(1 * dt)
        if pressed[K_PAGEDOWN]:
            self.graphics.zoomCamera(-1 * dt)

        mouse_pressed = pygame.mouse.get_pressed()
        mouse_rel = vec2d( pygame.mouse.get_rel() )
        ### Camera Drag
        if mouse_pressed[2]:
            v = mouse_rel / 750.0
            v.y *= -1
            self.graphics.moveCamera(v)
        mouse_pos = vec2d( pygame.mouse.get_pos() )
        
        
        ### Unmark old menu item
        for hit in self.hits:
            hit.color.rgba = 0.85,0.85,0.85,0.85
        ### See which menu object is marked
        self.graphics.picker.set_position(mouse_pos)
        
        self.graphics.rootNode.accept(self.graphics.picker)
        self.hits = self.graphics.picker.hits

        for hit in self.hits:
            hit.color.rgba = 1.0,1.0,1.0,1.0
        
        ### Mouse tool pointer
        p = self.toWorld(mouse_pos)
        self.graphics.mouse_pointer.translate = p.x, p.y, 0
        self.gameState.mouse_switch.switch(self.action)    
        if self.actor_to_remove != None:
            if self.actor_to_remove.color != None:
                self.actor_to_remove.color.rgba = 1.0,1.0,1.0,1.0
        p = self.toWorld(mouse_pos)
        if self.action == "remove":
            actor = self.actorManager.findClosestActor( p, 100 )
            if actor != None and actor.removable:
                self.actor_to_remove = actor
                if self.actor_to_remove.color != None:
                    self.actor_to_remove.color.rgba = 0.6,0.6,0.6,1.0
        ### Stretch
        self.handleStretch(mouse_pos)
        
        for event in pygame.event.get():
            
            if event.type is MOUSEBUTTONDOWN:
                gSound.playSound(self, "click")
                
            if event.type is QUIT:
                self.gameStateManager.popAllStates()
                return
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                self.toMenu()
                return
            elif event.type == KEYDOWN and event.key == K_m:
                self.toMenu()
                return
            elif event.type == KEYDOWN and event.key == K_PAUSE:
                self.gameStateManager.pushState( PauseState(self.gameStateManager, self.graphics) )
                return 
            elif event.type == KEYDOWN and event.key == K_h:
                self.toHelp()
                return 
            elif event.type == KEYDOWN and event.key == K_r:
                self.reset()
                return 
            elif event.type == KEYDOWN and event.key == K_g:
                g = vec2d(  mouse_pos[X] - self.graphics.screenSize[X]/2,
                            -mouse_pos[Y] + self.graphics.screenSize[Y]/2 ) 
                self.physics.setGravity(g)
                
            elif event.type == KEYDOWN and event.key == K_t:
                self.gameStateManager.slowmo = not self.gameStateManager.slowmo
                
            elif event.type == KEYDOWN and event.key == K_o:
                self.actorManager.loadLevel( self.nextLevel )
            elif event.type == KEYDOWN and event.key == K_p:
                self.actorManager.saveLevel( self.nextLevel )
                
            elif event.type == KEYDOWN and event.key == K_1: self.nextLevel = "level1"
            elif event.type == KEYDOWN and event.key == K_2: self.nextLevel = "level2"
            elif event.type == KEYDOWN and event.key == K_3: self.nextLevel = "level3"
            elif event.type == KEYDOWN and event.key == K_4: self.nextLevel = "level4"
            elif event.type == KEYDOWN and event.key == K_5: self.nextLevel = "level5"
            elif event.type == KEYDOWN and event.key == K_6: self.nextLevel = "level6"
            elif event.type == KEYDOWN and event.key == K_7: self.nextLevel = "level7"
            elif event.type == KEYDOWN and event.key == K_8: self.nextLevel = "level8"
            elif event.type == KEYDOWN and event.key == K_9: self.nextLevel = "level9"
            elif event.type == KEYDOWN and event.key == K_0: self.nextLevel = "level0"
                               
            elif event.type == MOUSEBUTTONDOWN and event.button == WHEEL_UP:
                self.graphics.zoomCamera(10 * dt)
            elif event.type == MOUSEBUTTONDOWN and event.button == WHEEL_DOWN:
                self.graphics.zoomCamera(-10 * dt)
            elif event.type == MOUSEBUTTONDOWN and event.button == RIGHT_MOUSE:
                ### no code here..
                pass
            elif event.type == MOUSEBUTTONDOWN and event.button == LEFT_MOUSE:
                for hit in self.hits:
                    if hit.type == "menu":
                        if hit.name == "(m)enu":
                            self.toMenu()
                            return
                        elif hit.name == "(r)eset":
                            self.reset()
                        elif hit.name == "(h)elp":
                            self.toHelp()
                    elif hit.type == "object":
                        self.action = hit.name 
                if self.hits == []:
                    p = self.toWorld(mouse_pos)
                    ca = self.actorManager.createActor
                    if self.action == "gravity1" or self.action == "gravity2":
                        ca( self.action, p, free_object = True)
                    elif self.action == "box":
                        ca("box", p, free_object = True, size=vec2d(10, 10), static = True)
                    elif self.action == "stretch":
                        self.stretchActor = ca("stretch", p, free_object = True, static = True)
                    elif self.action == "remove":
                       actor = self.actorManager.findClosestActor( p, 100 )
                       if actor is not None and actor.removable:
                           self.actorManager.destroyActor( actor ) 
                        
                
            elif event.type == MOUSEBUTTONUP and event.button == LEFT_MOUSE:
                if self.stretchActor != None:
                    self.actorManager.finishStretching( self.stretchActor )
                    self.stretchActor = None
#                for hit in self.graphics.picker.hits:
#                    if hit.type == "menu":
#                        print hit.name
#                    elif hit.type == "object":
#                        print hit.name
    
    def toGui(self, pos):            
        v = vec2d(  pos[X] - self.graphics.screenSize[X]/2,
                   -pos[Y] + self.graphics.screenSize[Y]/2 ) 
        return v
    def toWorld(self, pos):
        v = vec2d(  pos[X] - self.graphics.screenSize[X]/2,
                   -pos[Y] + self.graphics.screenSize[Y]/2 ) 
        x,y,z = self.graphics.world.camera.translate
        v -= vec2d(x,y)
        v /= self.graphics.zoom
        
        #print v, pos
        return v
                   
                   
                   