import random

import qgl

from Graphics import Graphics
from Physics import Physics
from ActorManager import ActorManager
from MainGameInput import MainGameInput
from GameStateManager import GameStateManager
from MainGameGui import MainGameGui
from Level import Level
from pymunk.vec2d import vec2d

from WinState import WinState

class MainGameState:
    def __init__(self, gameStateManager, graphics, physics):
        
        self.gameStateManager = gameStateManager
        self.graphics = graphics
        self.physics = physics
        self.blockUpdate = True
        self.blockDraw = True
        
        self.actorManager = ActorManager( self.graphics, self.physics, self )
        #self.gui = MainGameGui( self.graphics, self.physics )
        
        self.input = MainGameInput( gameStateManager, graphics, physics, self.actorManager, self )
        self.level = Level()
        self.graphics.setBackground(self.level.name)
    
        self.physics.setActorManager(self.actorManager)
    
        self.stat_texts = {}
        
        ###       
        ### GUI
        ###
        self.gui = self.graphics.main_gui
        ### GUI BG
        texture = self.graphics.createTexture("game_menu_bg")
        quad = self.graphics.createQuad(self.graphics.screenSize)
        bg = qgl.scene.Group(texture, quad)
        self.gui.add(bg)
        
        ### Balls left to spawn
        self.balls_left_text = t = self.graphics.createText("left:      x 12", "lousy")
        t.size = 400
        g = qgl.scene.Group( t )
        g.translate = (-375, 250, 0)
        self.gui.add( g )
        texture = self.graphics.createTexture("ball")
        quad = self.graphics.createQuad((32,32))
        g = qgl.scene.Group(texture, quad)
        g.translate = (-285, 275, 0)
        self.gui.add( g )
        ### Balls left to win
        self.balls_win_text = t = self.graphics.createText("to win:      x 12", "lousy")
        t.size = 400
        g = qgl.scene.Group( t )
        g.translate = (-375, 200, 0)
        self.gui.add( g )
        texture = self.graphics.createTexture("ball")
        quad = self.graphics.createQuad((32,32))
        g = qgl.scene.Group(texture, quad)
        g.translate = (-265, 225, 0)
        self.gui.add( g )
        ### Level name
        self.level_text = t = self.graphics.createText(self.level.name, "lousy")
        t.size = 2000
        self.level_text_color = c = self.graphics.createColor(1,1,1)
        g = qgl.scene.Group( c, t )
        g.translate = (-200, -100, 0)
        self.gui.add( g )
        
        
        ### Mouse Pointer Stuff
        self.mouse_switch = qgl.scene.Switch()
        self.graphics.mouse_pointer.add( self.mouse_switch )
        #self.graphics.gui.add( self.gui )
        ### Gravity Pointer
        texture = self.graphics.createTexture("arrow")
        quad = self.graphics.createQuad((128,128))
        self.gravity_pointer = qgl.scene.Group()
        self.gui.add( self.gravity_pointer )
        self.gravity_pointer.add(texture, quad)
        self.gravity_pointer.translate = (-330, -230, 0)
        ### Tools
        x = -150
        for name in ["stretch", "box", "gravity1", "gravity2", "remove"]:
            ### Tool object
            texture = self.graphics.createTexture(name)
            quad = self.graphics.createQuad((64,64))
            g = qgl.scene.Group()
            self.gui.add( g )
            c = qgl.scene.state.Color((0.7,0.7,0.7,0.7))
            g.color = c
            g.add(c, texture, quad)
            g.translate = (x, -250, 0)
            g.selectable = True
            g.name = name
            g.type = "object"
            ### Tool mouse pointer
            texture = self.graphics.createTexture(name)
            if name in ["stretch", "box", "remove"]:
                s = (24,24)
            else:
                s = (64,64)
            quad = self.graphics.createQuad(s)
            g = qgl.scene.Group()
            self.mouse_switch.switch(name)
            self.mouse_switch.add( g )
            g.add(texture, quad)
            g.name = name

            ### Tool text
            if name != "remove":
                text = self.graphics.createText("10", "lousy")
                text.size = 400
                g = qgl.scene.Group( text )
                self.stat_texts[name] = text
                g.translate = (x, -250, 0)
                self.gui.add( g )
                x += 80
        ### Menu
        y = -170
        for name in ["(m)enu", "(r)eset", "(h)elp"]:
            text = self.graphics.createText(name, "lousy")
            text.size = 400
            c = qgl.scene.state.Color((0.85,0.85,0.85,0.85))
            g = qgl.scene.Group( c, text )
            g.color = c
            g.selectable = True
            g.name = name
            g.type = "menu"
            g.translate = (270, y, 0)
            g.selectable = True
            self.gui.add( g )
            y -= 50
        
        
        self.graphics.rootNode.accept( self.graphics.compiler )
        #self.graphics.gui.accept( self.graphics.compiler )
        self.graphics.main_gui.disable()
        
    def reset(self):
        self.physics.setGravity( vec2d(0,-1) )
        self.loadLevel(self.level.name)
        #print "reset level.."
        
    def loadLevel(self, level):
        print "MGS: Loading", level
        self.level_text.text = level
        self.level_text_color.rgba = 1.0,1.0,1.0,1.0
        self.actorManager.loadLevel(level)
        self.physics.setGravity( vec2d(0,-1) )
        self.graphics.setZoom( 0.6 )
        self.level = Level(name=level)
        
    def createStuff(self):    
        size = 3
        for x in range(10):
            for y in range(10):
                data = ( vec2d(size+x+y, size+x+y), False)
                #self.actorManager.createActor("box", vec2d(x*25, y*25), data)
                #self.actorManager.createActor("box", vec2d(x*25, y*25), size=vec2d(size+x+y, size+x+y), static = False)
        
        
        size = 10
        for x in range(-3, 3):
            for y in range(-3, 3):
                data = ( vec2d(size+x+y, size+x+y), False)
                #self.actorManager.createActor("box", vec2d(x*25, y*25), data)
                self.actorManager.createActor("box", vec2d(x*100+20, y*100), size=vec2d(size, size), static = True)
        
    
    def update(self, dt):
        #self.graphics.setActiveBranch("main")
        
        self.physics.update( dt*0.5 )
        self.physics.update( dt*0.5 )
        
        self.actorManager.update( dt )
        
        #self.gui.update( dt )
        self.input.update( dt )
        
        for k in self.level.objects_left.keys():
            self.stat_texts[k].text = str(self.level.objects_left[k])
 
        ###GUI       
        r,g,b,a = self.level_text_color.rgba
        if a < 0.1: a = 0
        m = (dt * 30 * 0.982)
        if m > 0.99: m = 0.95
        self.level_text_color.rgba = 1.0,1.0,1.0, a * m

        self.gravity_pointer.angle = self.physics.getGravity().get_angle() + 270
        self.balls_left_text.text = "left:      x " + str(self.level.balls_left)
        self.balls_win_text.text = "to win:      x " + str(self.level.goal)
        ### WIN CONDITION
        if self.level.isComplete():
            print self.level.name, "completed!"
            state = WinState( self.gameStateManager, self.graphics, self.level )
            if self.level.next_level is None:
                self.graphics.main_gui.disable()
            self.gameStateManager.pushState( state )
                


    def pop(self): 
        #todo: remove all graphic-nodes from switch used by this state.   
        pass
        
        