import random
import pygame
from pygame.locals import *
from Graphics import Graphics
from GameStateManager import GameStateManager
from pymunk.vec2d import vec2d
from Sound import gSound


import qgl

X,Y,Z = 0,1,2
class MenuState:
    def __init__(self, gameStateManager, graphics):
        self.gameStateManager = gameStateManager
        self.graphics = graphics
        self.blockUpdate = True

        self.gui = qgl.scene.Group()
        self.graphics.gui.add(self.gui)
        
        texture = self.graphics.createTexture("menu_bg")
        quad = self.graphics.createQuad(self.graphics.screenSize)
        bg = qgl.scene.Group(texture, quad)
        self.gui.add(bg)
        self.switch = qgl.scene.Switch()
        self.main_menu = qgl.scene.Group()      
        self.switch.switch("main menu")
        self.switch.active = "main menu"
        self.switch.add(self.main_menu)
        self.gui.add(self.switch)
        ### Marker
        texture = self.graphics.createTexture("marker")
        quad = self.graphics.createQuad((50,50))
        self.marker = qgl.scene.Group(texture, quad)
        self.marker.translate = -130,50,0
        self.gui.add(self.marker)
        self.selected = "start"
        
        ### Main Menu
        ### Start Game
        text = self.graphics.createText("Start!", "lousy")
        text.size = 1000
        g = qgl.scene.Group( text )
        g.selectable = True
        g.translate = -100, 0, 0
        g.name = "start"
        self.main_menu.add(g)
        ### Select Game
        text = self.graphics.createText("Select Level", "lousy")
        text.size = 1000
        g = qgl.scene.Group( text )
        g.selectable = True
        g.translate = -200, -100, 0
        g.name = "select level"
        self.main_menu.add(g)
        ### Quit
        text = self.graphics.createText("Quit", "lousy")
        text.size = 1000
        g = qgl.scene.Group( text )
        g.selectable = True
        g.translate = -100, -200, 0
        g.name = "quit"
        self.main_menu.add(g)
        
        ### Levels
        self.switch.switch("level menu")
        self.switch.active = "level menu"
        self.level_menu = qgl.scene.Group()
        self.addLevels()
        self.switch.add(self.level_menu)
        ### Back in level menu
        text = self.graphics.createText("Back", "lousy")
        text.size = 700
        g = qgl.scene.Group( text )
        g.selectable = True
        g.translate = 150, -250, 0
        g.name = "back"
        self.level_menu.add(g)
        
        ### Compile
        self.graphics.gui.accept(self.graphics.compiler)
        self.switch.switch("main menu")
        self.switch.active = "main menu"
        self.graphics.main_gui.disable()
        
    def addLevels(self):
        self.levels = ["level1", "level2", "level3", "level4", "level5", 
                  "level6", "level7", "level8", "level9",
                  "extra"]
        
        i = 0
        x = -175
        y = 50
        for level in self.levels:
            text = self.graphics.createText(level, "lousy")
            text.size = 500
            g = qgl.scene.Group( text )
            g.selectable = True
            g.translate = x, y, 0
            g.name = level
            g.level_name = level
            self.level_menu.add(g)
            y -= 50
            i += 1
            if i > 6:
                y = 50
                x += 175
                i = 0
        
    def update(self, dt):
        mpos = pygame.mouse.get_pos()
        self.graphics.picker.set_position(mpos)
        #try:
        self.graphics.rootNode.accept(self.graphics.picker)
        #except Exception, e:
         #   e
         #   self.graphics.picker.hits = []
        for hit in self.graphics.picker.hits:
            if self.switch.active == "level menu":
                self.marker.translate = hit.translate[X] - 30, hit.translate[Y] + 40, 0
            else:
                self.marker.translate = hit.translate[X] - 30, hit.translate[Y] + 50, 0
            self.selected = hit.name
        for event in pygame.event.get():
            
            if event.type is MOUSEBUTTONDOWN:
                gSound.playSound(self, "click")
            
            if event.type is QUIT:
                self.gameStateManager.popAllStates()
                return
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                self.gameStateManager.popState()
                return
            elif event.type is MOUSEBUTTONDOWN and event.button == 1:
                if self.selected == "start":
                    self.gameStateManager.states[0].loadLevel("level1")
                    self.graphics.main_gui.enable()
                    self.gameStateManager.popState()
                    return
                elif self.selected == "quit":
                    self.gameStateManager.popAllStates()
                    return
                elif self.selected == "select level":
                    self.switch.switch("level menu")
                    self.switch.active = "level menu"
                    self.marker.translate = -280,100,0
                    self.selected = self.levels[0]
                elif self.selected == "back":
                    self.switch.switch("main menu")
                    self.switch.active = "main menu"
                    self.marker.translate = -130,50,0
                    self.selected = "start"
                else:
                    level = self.selected
                    self.gameStateManager.states[0].loadLevel(level)
                    self.graphics.main_gui.enable()
                    self.gameStateManager.popState()
                    return
            
    def pop(self):
        self.graphics.gui.remove(self.gui)
        
        