
import ctypes as ct
import pymunk._chipmunk as cp
import pymunk.util as u
from pymunk.vec2d import vec2d
import functools

X,Y,Z = 0,1,2
COLLTYPE_DEFAULT = 0


def onCollide(extra, shape1, shape2, contact_points, num_contact_points, data):
    #print dir(contact_points.contents)
    # This is wrong. It only gets the first contact point.
    # Add cast to array.
    contact = contact_points.contents.p.x, contact_points.contents.p.y

    actorManager = extra[1]
    #print "ids:", shape1.contents.id, shape2.contents.id, "#actors:", len(actorManager.actors)
    actor1 = actorManager.shape_to_actor_map[shape1.contents.id]
    actor2 = actorManager.shape_to_actor_map[shape2.contents.id]

    actor1.onCollide( actor2, contact_points, num_contact_points, data )
    actor2.onCollide( actor1, contact_points, num_contact_points, data )
   
    return True

f = cp.cpCollFunc(onCollide)
        
class Physics:
    def __init__(self):
        
        ### Physics stuff
        cp.cpInitChipmunk()
        self.space = cp.cpSpaceNew()
        self.space.contents.gravity = vec2d(0.0, -900.0)
        
        cp.cpSpaceResizeStaticHash( self.space, 50.0, 2000)
        cp.cpSpaceResizeActiveHash( self.space, 50.0, 100)
    
    
    
    def setActorManager(self, am):
        global f
        extra = [self, am]
        callback_w = functools.partial(onCollide, extra)
        f = cp.cpCollFunc(callback_w)
        cp.cpSpaceAddCollisionPairFunc( self.space, COLLTYPE_DEFAULT, COLLTYPE_DEFAULT, f, None)

    def remove( self, body, shapes):
        #print self.space
        for shape in shapes:
            #print "type", shape.contents.type
            #print "id", shape.contents.id
            cp.cpSpaceRemoveShape( self.space, shape )
            cp.cpShapeFree( shape )
            shape = None
        cp.cpSpaceRemoveBody( self.space, body)
        cp.cpBodyFree( body )
    
    def removeStatic( self, body, shapes):
        #print dir(self.space.contents)
        
        for s in shapes:
            #print "type", s.contents.type
            #print "id", s.contents.id
            cp.cpSpaceRemoveStaticShape( self.space, s )
            cp.cpShapeFree( s )
            s = None
        cp.cpSpaceRemoveBody( self.space, body )
        cp.cpBodyFree( body )        

           
    def createStaticPolyLine(self, points, pos = (0,0), e = 0 ):
        if len(points) < 2:
            return []
        shapes = []
        body = cp.cpBodyNew(1e100, 1e100)
        for i in range(len(points)-1):
            v1 = vec2d(points[i].x , points[i].y) + pos
            v2 = vec2d(points[i+1].x, points[i+1].y) + pos
            shape = cp.cpSegmentShapeNew(body, v1, v2, 0.0)
            shape.contents.u = 1.0
            shape.contents.collision_type = COLLTYPE_DEFAULT
            shape.contents.e = e
            cp.cpSpaceAddStaticShape( self.space, shape )
            shapes.append(shape)
        return body, shapes
        
    def createPolygon( self, points, poly_mass = 5.0, pos = (0,0)):
        p_num = len(points)
        P_ARR = vec2d * p_num
        p_arr = P_ARR(vec2d(0,0))
        
        xsum = 0
        ysum = 0

        for point in points:
            xsum += point[X]
            ysum += point[Y]
        points2 = []
        for point in points:
            x = point[X] - xsum / p_num
            y = point[Y] - ysum / p_num
            points2.append(vec2d(x,y))
        points = points2
        #print points2
        for i in range(p_num):
            p_arr[i].x = points[i][X] 
            p_arr[i].y = points[i][Y]
        pos = pos[X] + xsum / p_num , pos[Y] + ysum / p_num
        
        #print pos    
        poly_moment = cp.cpMomentForPoly(poly_mass, p_num, p_arr, vec2d(0,0))    
        poly_moment = 1000
        body = cp.cpBodyNew(poly_mass, poly_moment)
        body.contents.p = vec2d(pos[X], pos[Y])
        cp.cpSpaceAddBody( self.space, body )
        
        shape = cp.cpPolyShapeNew(body, p_num, p_arr, vec2d(0,0))
        shape.contents.u = 0.5
        shape.contents.collision_type = COLLTYPE_DEFAULT
        cp.cpSpaceAddShape( self.space, shape )
        return body, shape
    
    
    def createBall( self, point, ball_mass=1.0, ball_radius=15.0, static=False):
 
        ball_moment = cp.cpMomentForCircle(ball_mass, ball_radius, 0.0, vec2d(0,0) )
        ballBody = cp.cpBodyNew(ball_mass, ball_moment)
        ballBody.contents.p = vec2d(point[X],point[Y])

        if not static:
            cp.cpSpaceAddBody( self.space, ballBody ) 
        
        ballShape = cp.cpCircleShapeNew(ballBody, ball_radius, vec2d(0,0))
        ballShape.contents.u = 1.5
        ballShape.contents.e = 0.9
        ballShape.contents.collision_type = COLLTYPE_DEFAULT
        if static:
            cp.cpSpaceAddStaticShape( self.space, ballShape )
        else:
            cp.cpSpaceAddShape( self.space, ballShape )
        return ballBody, ballShape
    
    def setGravity(self, v):
        g = v.normalized()
        self.space.contents.gravity = g * 900
        
    def getGravity(self):
        return vec2d(self.space.contents.gravity)
    
    def applyImpulse(self, body, impulse, offset=vec2d(0,0)):
        #cp.cpBodyApplyImpulse(body, impulse, offset)
        body.contents.v += impulse * body.contents.m_inv
        body.contents.w += body.contents.i_inv * offset.cross( impulse)
    
    def applyForce(self, body, force, offset=vec2d(0,0)):
        cp.cpBodyApplyForce(body, force, offset)
    
    
    def setAngle(self, body, angle):
        cp.cpBodySetAngle( body, angle )
    
    def update(self, dt):
        
        cp.cpSpaceStep( self.space, dt)
        
    
    
    