import pygame
from pygame.locals import *
import ctypes as ct
import pymunk._chipmunk as cp
import pymunk.util as u
from pymunk.vec2d import vec2d

       
v = vec2d(1,1)
black = (0,0,0)
gray = (100,100,100)
lightgray = (200,200,200)
red = (255,0,0)
green = (0,255,0)
blue = (0,0,255)
white = (255,255,255)
X,Y,Z = 0,1,2

COLLTYPE_DEFAULT = 0
COLLTYPE_MOUSE = 1

contact = None
shape_to_remove = None

def cpvrotate(v1, v2):
    return vec2d(v1.x*v2.x - v1.y*v2.y, v1.x*v2.y + v1.y*v2.x)
def cpvadd(v1, v2):
    return vec2d(v1.x + v2.x, v1.y + v2.y)
def cpvzero():
    return vec2d(0,0)
def flipy(y):
    return -y+600
    
#cpCollFunc = CFUNCTYPE(c_int, POINTER(cpShape), POINTER(cpShape), POINTER(cpContact), c_int, c_void_p)    
def printMouseColl(shape1, shape2, contact_points, num_contact_points, data):
    #print dir(contact_points.contents)
    global contact
    global shape_to_remove
    # This is wrong. It only gets the first contact point.
    # Add cast to array.
    contact = contact_points.contents.p.x, contact_points.contents.p.y
   
    if shape1.contents.collision_type == COLLTYPE_DEFAULT:
        shape_to_remove = shape1
    elif shape2.contents.collision_type == COLLTYPE_DEFAULT:
        shape_to_remove = shape2
    return False


def createBall(space, point, ball_mass=1.0, ball_radius=15.0):

    ball_moment = cp.cpMomentForCircle(ball_mass, ball_radius, 0.0, cpvzero())
    ballBody = cp.cpBodyNew(ball_mass, ball_moment)
    ballBody.contents.p = vec2d(point[X],point[Y])
    cp.cpSpaceAddBody(space, ballBody)
    
    ballShape = cp.cpCircleShapeNew(ballBody, ball_radius, cpvzero())
    ballShape.contents.u = 1.5
    ballShape.contents.collision_type = COLLTYPE_DEFAULT
    cp.cpSpaceAddShape(space, ballShape)
    return ballShape

def createPoly(space, points, poly_mass = 5.0, pos = (0,0)):
    p_num = len(points)
    P_ARR = vec2d * p_num
    p_arr = P_ARR(vec2d(0,0))
    
    xsum = 0
    ysum = 0
    #print "createPoly"
    #print points
    for point in points:
        xsum += point[X]
        ysum += point[Y]
    points2 = []
    for point in points:
        x = point[X] - xsum / p_num
        y = point[Y] - ysum / p_num
        points2.append((x,y))
    points = points2
    #print points2
    for i in range(p_num):
        p_arr[i].x = points[i][X] 
        p_arr[i].y = points[i][Y]
    pos = pos[X] + xsum / p_num , pos[Y] + ysum / p_num
    
    #print pos    
    poly_moment = cp.cpMomentForPoly(poly_mass, p_num, p_arr, vec2d(0,0))    
    poly_moment = 1000
    body = cp.cpBodyNew(poly_mass, poly_moment)
    body.contents.p = vec2d(pos[X], pos[Y])
    cp.cpSpaceAddBody(space, body)
    
    shape = cp.cpPolyShapeNew(body, p_num, p_arr, vec2d(0,0))
    shape.contents.u = 0.5
    shape.contents.collision_type = COLLTYPE_DEFAULT
    cp.cpSpaceAddShape(space, shape)
    return shape

def createWall(space, points):
    if len(points) < 2:
        return []
    walls = []
    for i in range(len(points)-1):
        v1 = vec2d(points[i][X], points[i][Y])
        v2 = vec2d(points[i+1][X], points[i+1][Y])
        wallBody = cp.cpBodyNew(1e100, 1e100)
        wallShape = cp.cpSegmentShapeNew(wallBody, v1, v2, 0.0)
        wallShape.contents.u = 1.0
        wallShape.contents.collision_type = COLLTYPE_DEFAULT
        cp.cpSpaceAddStaticShape(space, wallShape)
        walls.append(wallShape)
    return walls
    
def main():
    
    global contact
    global shape_to_remove
        
    pygame.init()
    screen = pygame.display.set_mode((600, 600))
    clock = pygame.time.Clock()
    running = True
    
    ### Physics stuff
    cp.cpInitChipmunk()
    space = cp.cpSpaceNew()
    space.contents.gravity = vec2d(0.0, -900.0)
    
    cp.cpSpaceResizeStaticHash(space, 50.0, 2000)
    cp.cpSpaceResizeActiveHash(space, 50.0, 100)
    
    ### Walls
    walls = [] + createWall(space,[(100, 50), (500, 50)])
    
    ## Balls
    balls = []
    
    ### Polys
    polys = []
    ### Mouse
    mouseBody = cp.cpBodyNew(1e100, 1e100)
    p = pygame.mouse.get_pos()
    mouseBody.contents.p = vec2d(p[X],flipy(p[Y]))
    cp.cpSpaceAddBody(space, mouseBody)
    
    mouseShape = cp.cpCircleShapeNew(mouseBody, 3, cpvzero())
    mouseShape.contents.collision_type = COLLTYPE_MOUSE
    cp.cpSpaceAddShape(space, mouseShape)
    f = cp.cpCollFunc(printMouseColl)
    cp.cpSpaceAddCollisionPairFunc(space, COLLTYPE_DEFAULT, COLLTYPE_MOUSE, f, None)
    cp.cpSpaceAddCollisionPairFunc(space, COLLTYPE_MOUSE, COLLTYPE_DEFAULT, f, None)
    #cp.cpSpaceAddCollisionPairFunc(space, COLLTYPE_DEFAULT, COLLTYPE_DEFAULT, cp.cpCollFunc(printMouseColl), None)
    run_physics = True
    wall_points = []
    poly_points = []
    box_points = [(-18, -18), (-18, 18), (18,18), (18, -18)]

    t = 0
    shape_to_remove = None
    while running:
        
        contact = None
        
        for event in pygame.event.get():
            if event.type == QUIT:
                running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                running = False
            elif event.type == MOUSEBUTTONDOWN and event.button == 1:
                if pygame.key.get_mods() & KMOD_SHIFT:
                    p = event.pos[X], flipy(event.pos[Y])
                    polys.append(createPoly(space, box_points, pos = p))    
                else:
                    p = event.pos[X], flipy(event.pos[Y])
                    balls.append(createBall(space, p))
            elif event.type == MOUSEBUTTONDOWN and event.button == 3: 
                if pygame.key.get_mods() & KMOD_SHIFT:
                    p = event.pos[X], flipy(event.pos[Y])
                    poly_points.append(p)
                    
                elif pygame.key.get_mods() & KMOD_CTRL:
                    p = event.pos[X], flipy(event.pos[Y])
                    wall_points.append(p)
                elif shape_to_remove is not None:                   
                    balls = filter(lambda a: a.contents.id != shape_to_remove.contents.id, balls)
                    walls = filter(lambda a: a.contents.id != shape_to_remove.contents.id, walls)
                    polys = filter(lambda a: a.contents.id != shape_to_remove.contents.id, polys)
                    cp.cpSpaceRemoveBody(space, shape_to_remove.contents.body)
                    if (shape_to_remove.contents.type == 1):
                        cp.cpSpaceRemoveStaticShape(space, shape_to_remove)
                    else:
                        cp.cpSpaceRemoveShape(space, shape_to_remove)
                    
            elif event.type == KEYUP and event.key in (K_RCTRL, K_LCTRL):
                ### Create Wall
                walls += createWall(space, wall_points)
                wall_points = []
            elif event.type == KEYUP and event.key in (K_RSHIFT, K_LSHIFT):
                ### Create Polygon
                #print is_clockwise(poly_points)
                if not u.is_clockwise(poly_points):
                    poly_points.reverse()
                if len(poly_points) > 2:
                    polys.append(createPoly(space, poly_points))
                poly_points = []
            elif event.type == KEYDOWN and event.key == K_SPACE:    
                run_physics = not run_physics
            elif event.type == KEYDOWN and event.key == K_k:
                for x in range (-100,100,25):
                    for y in range(-100,100,25):
                        p = pygame.mouse.get_pos()
                        p = x + p[X], y + flipy(p[Y])
                        polys.append(createPoly(space, box_points, pos=p))                

        p = pygame.mouse.get_pos()
        mouseBody.contents.p = vec2d(p[X],flipy(p[Y]))
       
        ### Update physics
        if run_physics:
            shape_to_remove = None
            dt = 1.0/60.0 # Best if dt is static.
            for ball in balls:
                cp.cpBodyResetForces(ball.contents.body)
                pass
            for poly in polys:
                cp.cpBodyResetForces(poly.contents.body)
                pass
            

            cp.cpSpaceStep(space, dt)
            
        ### Draw stuff
        screen.fill(white)

        # Display some text
        font = pygame.font.Font(None, 16)
        text = """LMB: Create ball
LMB + Shift: Create box
RMB on object: Remove object
RMB + Shift: Create polygon, release to finish (MUST be convex)
RMB + Ctrl: Create wall, release to finish
Space: Stop physics simulation
k: Add a bunch of boxes at the mouse pointer position"""
        y = 5
        for line in text.splitlines():
            text = font.render(line, 1,black)
            screen.blit(text, (5,y))
            y += 10

        ### Balls
        xs = []
        for ball in balls:
            body = ball.contents.body
            shape = ct.cast(ball, ct.POINTER(cp.cpCircleShape))
            v = cpvadd(body.contents.p, cpvrotate(shape.contents.c, body.contents.rot))
            p = v.x, flipy(v.y)
            r = shape.contents.r
            pygame.draw.circle(screen, blue, p, r, 2)
            if body.contents.p.x < -1000 or body.contents.p.x > 1000 or body.contents.p.y < -1000 or body.contents.p.y > 1000:
                xs.append((ball, body))
            #print body.contents.p.x, body.contents.p.y
        for ball, body in xs:
            cp.cpSpaceRemoveShape(space, ball)
            cp.cpSpaceRemoveBody(space, body)
            balls.remove(ball)
            pass
        
        #print dir(terrainShape.contents)
        ### Walls
        for wall in walls:
            body = wall.contents.body;
            shape = ct.cast(wall, ct.POINTER(cp.cpSegmentShape))
            pv1 = cpvadd(body.contents.p, cpvrotate(shape.contents.a, body.contents.rot))
            pv2 = cpvadd(body.contents.p, cpvrotate(shape.contents.b, body.contents.rot))
            p1 = pv1.x, flipy(pv1.y)
            p2 = pv2.x, flipy(pv2.y)
            pygame.draw.lines(screen, lightgray, False, [p1,p2])
        
        ### Uncompleted walls
        if len(wall_points) > 1:
            ps = [(p[X],flipy(p[Y])) for p in wall_points]
            pygame.draw.lines(screen, gray, False, ps, 2)
            
        ### Polys  
        xs = []        
        for poly in polys:
            body = poly.contents.body
            shape = ct.cast(poly, ct.POINTER(cp.cpPolyShape))
            num = shape.contents.numVerts
            verts = shape.contents.verts
            ps = []
            for i in range(num):
                v = cpvadd(body.contents.p, cpvrotate(verts[i], body.contents.rot))
                ps.append((v.x, flipy(v.y)))
            v = cpvadd(body.contents.p, cpvrotate(verts[0], body.contents.rot))
            ps.append((v.x, flipy(v.y)))
            if u.is_clockwise(ps):
                c = green
            else:
                c = red
            pygame.draw.lines(screen, c, False, ps)
            if body.contents.p.x < -1000 or body.contents.p.x > 1000 \
                or body.contents.p.y < -1000 or body.contents.p.y > 1000:
                xs.append((poly, body))
        for poly, body in xs:
            cp.cpSpaceRemoveShape(space, poly)
            cp.cpSpaceRemoveBody(space, body)
            polys.remove(poly)

                
        ### Uncompleted poly
        if len(poly_points) > 1:
            ps = [(p[X],flipy(p[Y])) for p in poly_points]
            pygame.draw.lines(screen, red, False, ps, 2)
        
        ### Mouse Contact
        if contact is not None:
            p = contact[X], flipy(contact[Y])
            pygame.draw.circle(screen, red, p, 3)
            
        ### Flip screen
        pygame.display.flip()
        clock.tick(50)
        pygame.display.set_caption("fps: " + str(clock.get_fps()))
        
if __name__ == '__main__':
    doprof = 0
    if not doprof: 
        main()
    else:
        import cProfile, pstats
        
        prof = cProfile.run("main()", "profile.prof")
        stats = pstats.Stats("profile.prof")
        stats.strip_dirs()
        stats.sort_stats('cumulative', 'time', 'calls')
        stats.print_stats(30)