import pygame
from pygame.locals import *

from pymunk.vec2d import vec2d
from pymunk.util import *

black = (0,0,0)
gray = (100,100,100)
lightgray = (200,200,200)
red = (255,0,0)
green = (0,255,0)
blue = (0,0,255)
white = (255,255,255)
X,Y,Z = 0,1,2

def flipyv(v):
    v.y = -v.y+600
    return v

def flipy(y):
    return -y+600

def draw_text(screen):
    # Display some text
    font = pygame.font.Font(None, 16)
    text = """Hold LMB to draw"""
    y = 5
    for line in text.splitlines():
        text = font.render(line, 1,black)
        screen.blit(text, (5,y))
        y += 10
        
def draw_poly(screen, points, color = red, r = 1):
    font = pygame.font.Font(None, 18)
    if len(points) > 1:
        ps = [vec2d(p.x, flipy(p.y)) for p in points]
        pygame.draw.lines(screen, color, False, ps, r)
        i = 0
        for p in ps:
            pygame.draw.circle(screen, blue, p, 3, 1)   
            text = str((p.x, flipy(p.y))) + " " + str(i)
            i += 1           
    
def main():  
    pygame.init()
    screen = pygame.display.set_mode((600, 600))
    clock = pygame.time.Clock()
    running = True
    
    ### Polys
    polys = []
    poly_points = []
    hulls = []
    drawing = False
    # Just some points to have something on screen
    testps =  [(299.0, 310.0), (399.0, 352.0), (386.0, 397.0), (327.0, 360.0), (304.0, 341.0), (250.0, 410.0)]
    testps = map(vec2d, testps)
    testps = [p * 2 - vec2d(300,300) for p in testps]
    hulls.append(convex_hull(testps))
    polys.append(testps)
    i = 0
    while running:
        
        contact = None
        
        for event in pygame.event.get():
            if event.type == QUIT:
                running = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                running = False
            elif event.type == MOUSEBUTTONDOWN and event.button == 1:
                drawing = True
                poly_points = []
            elif event.type == MOUSEBUTTONUP and event.button == 1:
                drawing = False
                poly_points = reduce_poly(poly_points)
                if len(poly_points) > 2:
                    hull = convex_hull(poly_points)
                    polys.append(poly_points)
                    hulls.append(hull)
                poly_points = []                
        
        if drawing:# and i % 5 == 0:
            p = vec2d(pygame.mouse.get_pos())
            flipyv(p)
            poly_points.append(p)
        i+= 1
        ### Draw stuff
        screen.fill(white)
        draw_text(screen)
      
        ### Completed poly
        for poly in polys:
            draw_poly(screen, poly, color=green, r = 3)
        for hull in hulls:
            draw_poly(screen, hull + [hull[0]], color=black)
        ### Uncompleted poly
        draw_poly(screen, poly_points)
        
        ### Mouse Contact
        if contact is not None:
            p = contact[X], flipy(contact[Y])
            pygame.draw.circle(screen, red, p, 3)
        ### Flip screen
        pygame.display.flip()
        clock.tick(50)
        pygame.display.set_caption("fps: " + str(clock.get_fps()))
        
if __name__ == '__main__':
    doprof = 0
    if not doprof: 
        main()

    else:
        import cProfile, pstats
        
        prof = cProfile.run("main()", "profile.prof")
        stats = pstats.Stats("profile.prof")
        stats.strip_dirs()
        stats.sort_stats('cumulative', 'time', 'calls')
        stats.print_stats(30)